require_dependency "people/api/v1/application_controller"
require 'authorization'

module People
  class Api::V1::AuthenticationController < Api::V1::ApplicationController
    
    skip_before_filter :authenticate_user, :only => [:register, :login]
    before_action :register_authorize, only: [:register]
    before_action :login_authorize, only: [:login]
    before_action :logout_authorize, only: [:logout]

    # POST /api/1/authentication/register
    def register
      #Create a new user
      user = ::People::V1::User.new(user_params)
      if user.save
        #If the user is saved, return a token
        successful_login(user,user.tokens[0])
        #Send confirmation email here
        ::People::AccountMailer.welcome_email(user).deliver
        ::People::AccountMailer.email_confirmation(user).deliver
      else
        #Return an error if not saved
        render :json => {errors: user.errors.full_messages}, status: :unprocessable_entity
      end
    end

    # POST /api/1/authentication/login
    def login
      errors = "Email and/or Password is incorrect"
      user = ::People::V1::User.find_by(email: get_email)
      if is_locked?(user)
        return
      end
      if user && user.authenticate(params[:user][:password])
        successful_login(user,user.tokens.create)
      else
        if unsuccessful_login?(user)
          errors = "Account is locked due to too many failed login attempts"
        end
        render :json => {errors: errors}, status: :unauthorized 
      end
    end

    # POST /api/1/authentication/logout
    def logout
      #Destroy the current token
      token = current_token
      token.destroy
      render json: {}
    end

    # GET /api/1/authentication/login_status
    def login_status
      # Will give an unauthorized if not logged in
      render json: {}
    end
    
    # POST /api/1/authentication/authenticate
    def authenticate
      if authenticate_password
        render :json => {errors: "Email and/or Password is incorrect"}, status: :unauthorized 
        return
      end
      render json: {}
    end

    private

      # Only allow a trusted parameter "white list" through.

      def user_params
        params.require(:user).permit(:username, :email, :password, :password_confirmation)
      end

      def unsuccessful_login?(user)
        if !user.nil?
          user.attempts = user.attempts + 1
          if ::People::V1::User.over_max_attempts?(user.attempts)
            user.locked = true
            #Send locked email here
            ::People::AccountMailer.unlock_account(user).deliver
          end
          user.save
          return true
        else
          return false
        end
      end

      def successful_login(user,token)
        user.attempts = 0
        user.save
        #Could possibly create a remember me option to set this number
        ::Arcadex::Create.set_token(token,20000,request,nil)
        userHash = {id: user.id, username: user.username, email: user.email}
        tokenHash = {auth_token: token.auth_token}
        render :json => {user: userHash,token: tokenHash}
      end

      def get_email
        if !params[:user][:email].nil?
          return params[:user][:email].downcase
        else
          return nil
        end
      end

      def is_locked?(user)
        if !user.nil?
          if user.locked
            errors = "Account is locked"
            render :json => {errors: errors}, status: :unauthorized
            return true
          else
            return false
          end
        end
      end

      def register_authorize
        if !::Authorization::People::V1::Authentication.register?
          render :json => {errors: "User is not authorized for this action"}, status: :forbidden
        end
      end

      def login_authorize
        if !::Authorization::People::V1::Authentication.login?
          render :json => {errors: "User is not authorized for this action"}, status: :forbidden
        end
      end

      def logout_authorize
        if !::Authorization::People::V1::Authentication.logout?(current_user)
          render :json => {errors: "User is not authorized for this action"}, status: :forbidden
        end
      end
      
  end
end
