require_dependency "people/api/v1/application_controller"
require 'authorization'

module People
  class Api::V1::UsersController < Api::V1::ApplicationController
    
    before_action :set_user, only: [:show, :update, :destroy]
    before_action :index_authorize, only: [:index]
    before_action :show_authorize, only: [:show]
    before_action :update_authorize, only: [:update]
    before_action :destroy_authorize, only: [:destroy]
    before_action :user_authenticate, only: [:update, :destroy]

    # GET /api/1/users
    def index
      @users = ::People::V1::User.all
      render json: @users
    end

    # GET /api/1/users/1
    def show
      render json: @user
    end

    # PATCH/PUT /api/1/users/1
    def update
      change_password
      before_email = @user.email
      if @user.update(user_params)
        if email_changed?(before_email.downcase,@user.email.downcase)
          #The email changed, reconfirm email address
          ::People::AccountMailer.email_confirmation(@user).deliver
        end
        render json: @user
      else
        render :json => {errors: @user.errors.full_messages}, status: :unprocessable_entity
      end
    end

    # DELETE /api/1/users/1
    def destroy
      @user.destroy
      render json: {}
    end

    private

      # Only allow a trusted parameter "white list" through.

      def user_params
        params.require(:user).permit(:username, :email, :password, :password_confirmation)
      end

      def user_authenticate
        if authenticate_password
          render :json => {errors: "Current password is incorrect"}, status: :unauthorized 
          return
        end
      end

      def email_changed?(old_email,new_email)
        if old_email != new_email
          @user.confirmed = false
          @user.save
          #Delete all previous emails
          ::People::EmailHash.delete_all(:user_id => ["user_id = ?", @user.id])
          return true 
        else
          return false
        end
      end

      #If password is being changed, need to change it over to fit user_params
      #Need new_password and new_password_confirmation to change password
      def change_password
        if params[:user].nil?
          return
        end
        if !params[:user][:new_password].nil? && !params[:user][:new_password_confirmation].nil?
          params[:user][:password] = params[:user][:new_password]
          params[:user][:password_confirmation] = params[:user][:new_password_confirmation]
        end
      end

      # Use callbacks to share common setup or constraints between actions.

      def set_user
        @user = ::People::V1::User.find_by_id(params[:id])
        if @user.nil?
          render :json => {errors: "User was not found"}, status: :not_found
        end
      end

      # Authorizations below here

      def index_authorize
        if !::Authorization::People::V1::User.index?(current_user)
          render :json => {errors: "User is not authorized for this action"}, status: :forbidden
        end
      end

      def show_authorize
        if !::Authorization::People::V1::User.show?(@user,current_user)
          render :json => {errors: "User is not authorized for this action"}, status: :forbidden
        end
      end

      def update_authorize
        if !::Authorization::People::V1::User.update?(@user,current_user)
          render :json => {errors: "User is not authorized for this action"}, status: :forbidden
        end  
      end

      def destroy_authorize
        if !::Authorization::People::V1::User.destroy?(@user,current_user)
          render :json => {errors: "User is not authorized for this action"}, status: :forbidden
        end  
      end
  end
end
