require "rails_helper"

#-#-#-#-#Collection Routes#-#-#-#-#
RSpec.describe ::People::V1::User, :type => :request do
  describe "Register" do
    before(:example) do
      @attrs = FactoryGirl.attributes_for(:people_user_1)
      @header = {::Settings.main_api_header => ::Settings.main_api_key}
    end
    # post /api/1/authentication/register
    it "checks response of a register request with a valid user object" do
      hash = {"user" => @attrs}
      post 'api/1/authentication/register', hash, @header
      expect(response.status).to eq(200) #ok
      expect(::People::V1::User.all.count).to eq(1)
      #A valid token needs to be returned
      expect(json["token"]["auth_token"]).to_not eq(nil)
      token = ::People::V1::User.find(1).tokens[0]
      expect(json["token"]["auth_token"]).to eq(token.auth_token)
    end
    # post /api/1/authentication/register
    it "checks response of a register request with mismatched passwords" do
      @attrs["password"] = "password1"
      @attrs["password_confirmation"] = "password2"
      hash = {"user" => @attrs}
      post 'api/1/authentication/register', hash, @header
      expect(response.status).to eq(422) #invalid_resource
      expect(::People::V1::User.all.count).to eq(0)
      #Errors need to be returned
      expect(json["errors"]).to_not eq(nil)
    end
  end
end
RSpec.describe ::People::V1::User, :type => :request do
  describe "Login" do
    before(:example) do
      @attrs = FactoryGirl.attributes_for(:people_user_1)
      @header = {::Settings.main_api_header => ::Settings.main_api_key}
    end
    # post /api/1/authentication/login
    it "checks response of a valid login request" do
      @attrs["password"] = "password123"
      @attrs["password_confirmation"] = "password123"   
      user = FactoryGirl.create(:people_user_1,@attrs)
      old_auth_token = user.tokens[0].auth_token
      #It needs to send an email and password
      hash = {"user" => @attrs}
      post 'api/1/authentication/login', hash, @header
      expect(response.status).to eq(200) #ok
      #A valid and new token need to be returned
      expect(json["token"]["auth_token"]).to_not eq(old_auth_token)
      #This token should be the newest token in the database
      new_auth_token = ::People::V1::User.find(1).tokens.order("created_at").last.auth_token
      expect(json["token"]["auth_token"]).to eq(new_auth_token)
    end
    # post /api/1/authentication/login
    it "checks response of an invalid login request with an invalid password" do
      @attrs["password"] = "password123"
      @attrs["password_confirmation"] = "password123"   
      user = FactoryGirl.create(:people_user_1,@attrs)
      #The password needs to be invalid
      @attrs["password"] = "wrongPassword"
      hash = {"user" => @attrs}
      #It needs to send an email and password
      post 'api/1/authentication/login', hash, @header
      expect(response.status).to eq(401) #unauthorized
      #Errors need to be returned
      expect(json["errors"]).to_not eq(nil)
    end
    # post /api/1/authentication/login
    it "check if a user is locked out" do
      @attrs["password"] = "password123"
      @attrs["password_confirmation"] = "password123"   
      user = FactoryGirl.create(:people_user_1,@attrs)
      expect(user.locked).to eq(false)
      #It needs to send an email and password
      @attrs["password"] = "false_password"
      hash = {"user" => @attrs}
      while(!::People::V1::User.over_max_attempts?(user.attempts))
        post 'api/1/authentication/login', hash, @header
        user.reload
        expect(response.status).to eq(401) #unauthorized
      end
      expect(user.locked).to eq(true)
      #User is locked so this shouldnt work
      token = user.tokens[0].auth_token
      header = {::Settings.token_header => token, "Email" => user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      get "api/1/authentication/login_status", nil, header
      expect(response.status).to eq(401) #unauthorized
    end
  end
end
RSpec.describe ::People::V1::User, :type => :request do
  describe "Logout" do
    before(:example) do
      @attrs = FactoryGirl.attributes_for(:people_user_1)
    end
    # get /api/1/authentication/1/logout
    it "checks logout response and makes sure token is deleted" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      token = user.tokens[0].auth_token
      header = {::Settings.token_header => token, "Email" => user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      post 'api/1/authentication/logout', nil, header
      expect(::People::V1::User.count).to eq(1)
      expect(response.status).to eq(200) #ok
      expect(::Arcadex::Token.count).to eq(0)
    end
  end
end
RSpec.describe ::People::V1::User, :type => :request do
  describe "Authenticate" do
    before(:example) do
      @attrs = FactoryGirl.attributes_for(:people_user_1)
    end
    # post /api/1/authentication/authenticate
    it "checks a valid authenticate request" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      token = user.tokens[0].auth_token
      header = {::Settings.token_header => token, "Email" => user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      hash = {"user" => @attrs}
      post "api/1/authentication/authenticate", hash, header
      expect(response.status).to eq(200) #ok
    end
    # post /api/1/authentication/authenticate
    it "checks an invalid authenticate request" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      token = user.tokens[0].auth_token
      header = {::Settings.token_header => token, "Email" => user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      @attrs["password"] = "false_password"
      hash = {"user" => @attrs}
      post "api/1/authentication/authenticate", hash, header
      expect(response.status).to eq(401) #unauthorized
    end
  end
end
RSpec.describe ::People::V1::User, :type => :request do
  describe "Login_Status" do
    before(:example) do
      @attrs = FactoryGirl.attributes_for(:people_user_1)
    end
    # get /api/1/authentication/login_status
    it "returns 200 okay if token is valid" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      token = user.tokens[0].auth_token
      header = {::Settings.token_header => token, "Email" => user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      get "api/1/authentication/login_status", nil, header
      expect(response.status).to eq(200) #ok
    end
    # get /api/1/authentication/login_status
    it "returns a 401 if token is not valid" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      token = user.tokens[0].auth_token + "not_valid"
      header = {::Settings.token_header => token, "Email" => user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      get "api/1/authentication/login_status", nil, header
      expect(response.status).to eq(401) #unauthorized
    end
    # get /api/1/authentication/login_status
    it "returns a 401 if token does not exist" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      token = user.tokens[0].auth_token
      header = {"Email" => user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      get "api/1/authentication/login_status", nil, header
      expect(response.status).to eq(401) #unauthorized
    end
  end
end