require "rails_helper"

#-#-#-#-#Collection Routes#-#-#-#-#
RSpec.describe ::People::V1::User, :type => :request do
  describe "Mail sent during authentication" do
    before(:example) do
      @attrs = FactoryGirl.attributes_for(:people_user_1)
      @hash_attrs = FactoryGirl.attributes_for(:people_email_hash)
      @header = {::Settings.main_api_header => ::Settings.main_api_key}
    end
    it "checks that welcome and email confirm are sent after registering" do
      beforeCount = ::ActionMailer::Base.deliveries.count
      hash = {"user" => @attrs}
      post 'api/1/authentication/register', hash, @header 
      expect(response.status).to eq(200) #ok
      expect(::People::EmailHash.all.count).to eq(1)
      expect(::ActionMailer::Base.deliveries.count).to eq(beforeCount + 2)
    end
    it "checks that a locked out email is sent" do 
      user = FactoryGirl.create(:people_user_1,@attrs)
      expect(user.locked).to eq(false)
      @attrs["password"] = "false_password"
      hash = {"user" => @attrs}
      beforeCount = ::ActionMailer::Base.deliveries.count
      while(!::People::V1::User.over_max_attempts?(user.attempts))
        post 'api/1/authentication/login', hash, @header
        user.reload
        expect(response.status).to eq(401) #unauthorized
      end
      expect(user.locked).to eq(true)
      expect(::People::EmailHash.all.count).to eq(1)
      expect(::ActionMailer::Base.deliveries.count).to eq(beforeCount + 1)
    end
    it "checks that welcome email and email confirm are sent to the right person" do
      email = "myemail@aol.com"
      @attrs["email"] = email
      hash = {"user" => @attrs}
      post 'api/1/authentication/register', hash, @header
      lastMail = ::ActionMailer::Base.deliveries.last
      expect(lastMail.to[0]).to eq(email)
      ::ActionMailer::Base.deliveries.delete(lastMail)
      welcomeMail = ::ActionMailer::Base.deliveries.last
      expect(welcomeMail.to[0]).to eq(email)
    end
    it "checks that forgotten password sends an email" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      email = user.email
      attrs = {"email" => email}
      beforeCount = ::ActionMailer::Base.deliveries.count
      post 'account/posted_email', attrs
      expect(::ActionMailer::Base.deliveries.count).to eq(beforeCount + 1)
      lastMail = ::ActionMailer::Base.deliveries.last
      expect(lastMail.to[0]).to eq(email)
      expect(::People::EmailHash.all.count).to eq(1)
    end
    it "makes sure a user can confirm their email" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      @hash_attrs["user_id"] = user.id
      @hash_attrs["email_type"] = ::People::EmailHash.email_confirm
      hash = FactoryGirl.create(:people_email_hash,@hash_attrs)
      expect(user.confirmed).to eq(false)
      expect(::People::EmailHash.all.count).to eq(1)
      get "account/email_confirm?eh_id=#{hash.id}&token=#{hash.token}"
      expect(::People::EmailHash.all.count).to eq(0)
      user.reload
      expect(user.confirmed).to eq(true)
    end
    it "makes sure an email is sent after an email is confirmed" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      @hash_attrs["user_id"] = user.id
      @hash_attrs["email_type"] = ::People::EmailHash.email_confirm
      hash = FactoryGirl.create(:people_email_hash,@hash_attrs)
      expect(user.confirmed).to eq(false)
      expect(::People::EmailHash.all.count).to eq(1)
      beforeCount = ::ActionMailer::Base.deliveries.count
      get "account/email_confirm?eh_id=#{hash.id}&token=#{hash.token}"
      expect(::People::EmailHash.all.count).to eq(0)
      expect(::ActionMailer::Base.deliveries.count).to eq(beforeCount + 1)
      user.reload
      expect(user.confirmed).to eq(true)
    end
    it "makes sure a user can reset their password" do
      @attrs["password"] = "password123"
      @attrs["password_confirmation"] = "password123"
      user = FactoryGirl.create(:people_user_1,@attrs)
      expect(user.authenticate(@attrs["password"])).to_not eq(false)
      @hash_attrs["user_id"] = user.id
      @hash_attrs["email_type"] = ::People::EmailHash.forgotten_password
      hash = FactoryGirl.create(:people_email_hash,@hash_attrs)
      expect(::People::EmailHash.all.count).to eq(1)
      passwords = {"password" => "new_password", "password_confirmation" => "new_password"}
      post "account/change_password?eh_id=#{hash.id}&token=#{hash.token}", passwords
      expect(::People::EmailHash.all.count).to eq(0)
      user.reload 
      expect(user.authenticate("new_password")).to_not eq(false)
      expect(user.confirmed).to eq(true)
    end
    it "makes sure an email is sent after resetting a password" do
      @attrs["password"] = "password123"
      @attrs["password_confirmation"] = "password123"
      user = FactoryGirl.create(:people_user_1,@attrs)
      expect(user.authenticate(@attrs["password"])).to_not eq(false)
      @hash_attrs["user_id"] = user.id
      @hash_attrs["email_type"] = ::People::EmailHash.forgotten_password
      hash = FactoryGirl.create(:people_email_hash,@hash_attrs)
      expect(::People::EmailHash.all.count).to eq(1)
      beforeCount = ::ActionMailer::Base.deliveries.count
      passwords = {"password" => "new_password", "password_confirmation" => "new_password"}
      post "account/change_password?eh_id=#{hash.id}&token=#{hash.token}", passwords
      expect(::People::EmailHash.all.count).to eq(0)
      expect(::ActionMailer::Base.deliveries.count).to eq(beforeCount + 1)
      user.reload 
      expect(user.authenticate("new_password")).to_not eq(false)
      expect(user.confirmed).to eq(true)
    end
    it "makes sure all of a users email_hashes are gone after changing password" do
      @attrs["password"] = "password123"
      @attrs["password_confirmation"] = "password123"
      user = FactoryGirl.create(:people_user_1,@attrs)
      expect(user.authenticate(@attrs["password"])).to_not eq(false)
      @hash_attrs["user_id"] = user.id
      @hash_attrs["email_type"] = ::People::EmailHash.forgotten_password
      hash = FactoryGirl.create(:people_email_hash,@hash_attrs)
      FactoryGirl.create_list(:people_email_hash,9,@hash_attrs)
      @hash_attrs["email_type"] = ::People::EmailHash.email_confirm
      FactoryGirl.create_list(:people_email_hash,5,@hash_attrs)
      @hash_attrs["user_id"] = user.id + 1
      #Not owned by the user we care about
      FactoryGirl.create_list(:people_email_hash,5,@hash_attrs)
      expect(::People::EmailHash.all.count).to eq(20)
      passwords = {"password" => "new_password", "password_confirmation" => "new_password"}
      post "account/change_password?eh_id=#{hash.id}&token=#{hash.token}", passwords
      user.reload 
      expect(user.authenticate("new_password")).to_not eq(false)
      expect(user.confirmed).to eq(true)
      expect(::People::EmailHash.all.count).to eq(5)
    end
    it "makes sure all previous email confirm email hashes are gone after confirming your email" do
      user = FactoryGirl.create(:people_user_1,@attrs)
      @hash_attrs["user_id"] = user.id
      @hash_attrs["email_type"] = ::People::EmailHash.email_confirm
      hash = FactoryGirl.create(:people_email_hash,@hash_attrs)
      FactoryGirl.create_list(:people_email_hash,4,@hash_attrs)
      @hash_attrs["email_type"] = ::People::EmailHash.forgotten_password
      FactoryGirl.create_list(:people_email_hash,5,@hash_attrs)
      expect(::People::EmailHash.all.count).to eq(10)
      get "account/email_confirm?eh_id=#{hash.id}&token=#{hash.token}"
      expect(::People::EmailHash.all.count).to eq(5)
    end
    it "makes sure all email hashes are gone after changing your email" do
      @user = FactoryGirl.create(:people_user_1)
      token = @user.tokens[0].auth_token
      @header = {::Settings.token_header => token, "Email" => @user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      @attrs = FactoryGirl.attributes_for(:people_user_1)
      @hash_attrs["user_id"] = @user.id
      @hash_attrs["email_type"] = ::People::EmailHash.email_confirm
      FactoryGirl.create_list(:people_email_hash,5,@hash_attrs)
      @hash_attrs["email_type"] = ::People::EmailHash.forgotten_password
      FactoryGirl.create_list(:people_email_hash,5,@hash_attrs)
      @hash_attrs["user_id"] = @user.id + 1
      @hash_attrs["email_type"] = ::People::EmailHash.email_confirm
      FactoryGirl.create_list(:people_email_hash,5,@hash_attrs)
      @attrs["email"] = @user.email
      @attrs["username"] = @user.username
      @attrs["email"] = "Cole@LaunchU.com"
      hash = {"user" => @attrs}
      #This uses users 1 header
      put "api/1/users/#{@user.id}", hash, @header
      expect(response.status).to eq(200) #ok
      expect(json["user"]["email"]).to eq("cole@launchu.com")
      expect(::People::EmailHash.all.count).to eq(6)
    end
  end
end