require "rails_helper"

#-#-#-#-#REST#-#-#-#-#
RSpec.describe ::People::V1::User, :type => :request do
  describe "Index" do
    before(:example) do
      @user = FactoryGirl.create(:people_user_1)
      token = @user.tokens[0].auth_token
      @header = {::Settings.token_header => token, "Email" => @user.email, ::Settings.main_api_header => ::Settings.main_api_key}
    end
    # get /api/1/users
    it "checks index json response" do
      FactoryGirl.create_list(:people_user_1, 10)
      get 'api/1/users', nil, @header
      expect(response.status).to eq(200) #ok
      expect(json["users"]).to_not eq(nil)
    end
  end
end
RSpec.describe ::People::V1::User, :type => :request do
  describe "Show" do
    before(:example) do
      @user = FactoryGirl.create(:people_user_1)
      token = @user.tokens[0].auth_token
      @header = {::Settings.token_header => token, "Email" => @user.email, ::Settings.main_api_header => ::Settings.main_api_key}
    end
    # get /api/1/users/1
    it "checks show json response" do
      get "api/1/users/#{@user.id}", nil, @header
      expect(response.status).to eq(200) #ok
      expect(json["user"]["id"]).to eq(1)
    end
  end
end
RSpec.describe ::People::V1::User, :type => :request do
  describe "Update" do
    before(:example) do
      @user = FactoryGirl.create(:people_user_1)
      token = @user.tokens[0].auth_token
      @header = {::Settings.token_header => token, "Email" => @user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      @attrs = FactoryGirl.attributes_for(:people_user_1)
      @attrs["email"] = @user.email
      @attrs["username"] = @user.username
    end
    # put /api/1/users/1
    it "checks update json response" do
      @attrs["email"] = "Cole@LaunchU.com"
      hash = {"user" => @attrs}
      #This uses users 1 header
      put "api/1/users/#{@user.id}", hash, @header
      expect(response.status).to eq(200) #ok
      expect(json["user"]["email"]).to eq("cole@launchu.com")
    end
    it "makes sure a user cannot alter another users information" do
      FactoryGirl.create(:people_user_1)
      @attrs["email"] = "Cole@LaunchU.com"
      hash = {"user" => @attrs}
      #This uses users 1 header and user 2 id
      put "api/1/users/#{@user.id + 1}", hash, @header
      expect(response.status).to eq(403) #forbidden
    end
    it "makes sure a user can't update to bad values" do
      @attrs["email"] = ""
      hash = {"user" => @attrs}
      #This uses users 1 header
      put "api/1/users/#{@user.id}", hash, @header
      expect(response.status).to eq(422) #ok
    end
    # put /api/1/users/1
    it "makes sure a user can't update without sending the password" do
      @attrs["password"] = ""
      hash = {"user" => @attrs}
      #This uses users 1 header
      put 'api/1/users/1', hash, @header
      expect(response.status).to eq(401) #ok
    end
    # put /api/1/users/1
    it "makes sure a user can change their password" do
      @attrs["email"] = @user.email
      @attrs["new_password"] = "new"
      @attrs["new_password_confirmation"] = "new"
      hash = {"user" => @attrs}
      #This uses users 1 header
      put "api/1/users/#{@user.id}", hash, @header
      expect(response.status).to eq(200) #ok
      @attrs["password"] = "new"
      hash = {"user" => @attrs}
      post "api/1/authentication/authenticate", hash, @header
      expect(response.status).to eq(200) #ok
    end
    # put /api/1/users/1
    it "makes sure a user account is not confirmed after changing emails" do
      @user.confirmed = true
      @user.save
      expect(@user.confirmed).to eq(true)
      @attrs["email"] = "Cole@LaunchU.com"
      hash = {"user" => @attrs}
      #This uses users 1 header
      put "api/1/users/#{@user.id}", hash, @header
      expect(response.status).to eq(200) #ok
      @user.reload
      expect(@user.confirmed).to eq(false)
      expect(json["user"]["email"]).to eq("cole@launchu.com")
    end
  end
end
RSpec.describe ::People::V1::User, :type => :request do
  describe "Destroy" do
    before(:example) do
      @user = FactoryGirl.create(:people_user_1)
      token = @user.tokens[0].auth_token
      @header = {::Settings.token_header => token, "Email" => @user.email, ::Settings.main_api_header => ::Settings.main_api_key}
      @attrs = FactoryGirl.attributes_for(:people_user_1)
      @attrs["email"] = @user.email
      @attrs["username"] = @user.username
    end
    # delete /api/1/users/1
    it "makes sure a user can delete their accout" do
      hash = {"user" => @attrs}
      #This uses users 1 header
      delete "api/1/users/#{@user.id}", hash, @header
      expect(response.status).to eq(200) #ok
      expect(::People::V1::User.count).to eq(0)
    end
    # delete /api/1/users/1
    it "makes sure a user can't delete their account with the incorrect password" do
      @attrs["password"] = "Wrong_password"
      hash = {"user" => @attrs}
      #This uses users 1 header
      delete "api/1/users/#{@user.id}", hash, @header
      expect(response.status).to eq(401) #ok
      expect(::People::V1::User.count).to eq(1)
    end
    # delete /api/1/users/2
    it "makes sure a user can't delete another persons account" do
      @attrs["password"] = "Wrong_password"
      hash = {"user" => @attrs}
      #This uses users 1 header
      user2 = FactoryGirl.create(:people_user_1)
      delete "api/1/users/#{user2.id}", hash, @header
      expect(response.status).to eq(403) #ok
      expect(::People::V1::User.count).to eq(2)
    end
  end
end
#-#-#-#-#Serialization#-#-#-#-#
RSpec.describe ::People::V1::User, :type => :request do
  describe "Serialization" do
    before(:example) do
      @user = FactoryGirl.create(:people_user_1)
      token = @user.tokens[0].auth_token
      @header = {::Settings.token_header => token, "Email" => @user.email, ::Settings.main_api_header => ::Settings.main_api_key}
    end
    # get /api/1/users
    it "checks only appropriate attributes are sent back for index" do
      FactoryGirl.create_list(:people_user_1, 10)
      get 'api/1/users', nil, @header
      expect(response.status).to eq(200) #ok
      expect(::People::V1::User.count).to eq(11)
      expect(::Arcadex::Token.count).to eq(11)
      expect(json["users"][0]["password_digest"]).to eq(nil)
      expect(json["users"][0]["created_at"]).to_not eq(nil)
      expect(json["users"][0]["updated_at"]).to_not eq(nil)
    end
  end
end
#-#-#-#-#Errors#-#-#-#-#
RSpec.describe ::People::V1::User, :type => :request do
  describe "Authorization" do
    before(:example) do
      @user = FactoryGirl.create(:people_user_1)
      token = @user.tokens[0].auth_token
      @header = {::Settings.token_header => token, "Email" => @user.email, ::Settings.main_api_header => ::Settings.main_api_key}
    end
    it "checks for 404 response" do
      get "api/1/users/#{@user.id + 10}", nil, @header
      expect(response.status).to eq(404) #not_found
    end
  end
end